%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main program for Dynamics of Externalities: A second order perspective.
% call: derive.m esolve.m transformer.m easyirf2.m 
%       eei1.m eei2.m ee1.m ee2.m  eigvalue.m
% Huabin Wu. Dec. 2008. Revised on Jan. 2009, Apr. 2011. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
%================MODEL SET UP================
plotirf   = 1;  %<<<--- 1 if plot IRFs.
plotcoefs = 1;  %<<<--- 1 if plot the coefficients for second order terms.
ploteig   = 0;  %<<<--- 1 if plot eigenvalues.

% accuracy tests are time consuming!
atest     = 0;  %<<<--- 1 if draw the accuracy test figure.
atesteta  = 0;  %<<<--- 1 if draw the accuracy test figure wrt eta.
sig       = .3; % std. dev. of shocks.

% 1. parameters
syms     tao alf bta rho gma dlt eta
param = [tao alf bta rho gma dlt eta];

% 2. variables
txt={'A', 'k', 'Consumption','Labor', 'Investment', 'Output'};
syms  a  k  c  l  I  y ap kp cp lp Ip yp;
v = [a  k  c  l  I  y];
vp= [ap kp cp lp Ip yp]; 

nv=length(v);
idc=3:6; % indeces for controls
idx=1;   % shock index 

% 3. Equilibrium system
a1=alf*(1+eta);b1=(1-alf)*(1+eta);

f(1) = exp(l)^(1+gma-b1)-(1-alf)*exp(c)^(-tao)*exp(a)*(exp(k))^(a1);
f(2) = exp(c)^(-tao)-bta*exp(cp)^(-tao)*(alf *exp(yp)/exp(kp)+1-dlt);
f(3) = exp(c) + exp(kp) - (1-dlt) * exp(k)-exp(y);
f(4) = exp(y)-exp(a)*exp(k)^a1*exp(l)^b1;
f(5) = exp(I)+exp(c)-exp(y);
f(6) = ap-rho*a;

% 4. Calibration and Steady state 
alf=0.3; % capital share
bta=0.99;% discount rate
gma=0;   % labor elasticity
dlt=.025;% depreciation rate 
rho=0.9; % shock persistence
tao=1;   % consumption elasticity

eta=0.4; % level of externality

a1=alf*(1+eta);b1=(1-alf)*(1+eta);

ky = (bta*alf)/(1-bta*(1-dlt)); cy = 1-dlt*ky; %c/y ratio

l= ((1-alf)/cy)^(1/(1+gma));
k=(l^b1*ky)^(1/(1-a1)); 
y = k/ky; c = cy*y; I = y-c;a = 1; 

a = log(a); k = log(k); l = log(l); 
c = log(c); y = log(y); I = log(I);

%================SOLVING THE MODEL================
% 1. Derive the second order approximation of the equilibruim system

symb=[v,param];nsymb=eval(symb);

[A1, B1, B2, B2p] = derive(f,v,vp,symb,nsymb);

% 2. Solve the system by linear method based on LS.
[S S2 Ss C C2 Cs] = esolve(A1, B1, B2, B2p, idc, idx);
indet=0; % inditerminacy indicator
if isempty(S)
    indet=1;
    disp('Indeterminacy! Set sunspot shock as default.');
    idc=4:6;idx=[1 3];
    [S S2 Ss C C2 Cs]=esolve(A1, B1, B2, B2p, idc, idx);
end
S=real(S);S2=real(S2);Ss=real(Ss);
C=real(C);C2=real(C2);Cs=real(Cs);
% coefs=[S S2 Ss; C C2 Cs];
% disp('Coefficients')
% disp(num2str(coefs,' %6.4f'));

%===========GENERATING IMPULSE RESPONSES=============
% Order of state   : a k
% Order of control : c l I y

idshock=1; 
nc=length(idc);ns=nv-nc;nx=length(idx);

% Rearrange the coefficient matrices.
[xL xC]=transformer(ns);[xl xc]=transformer(nx);
T=eye(ns)+ones(ns);T=xC*T(:);
SS2 = reshape(xL*(S2'.*(T*ones(1,ns))),ns,ns,ns);
CS2 = reshape(xL*(C2'.*(T*ones(1,nc))),ns,ns,nc);

SIG=eye(nx); SIG=xc*SIG(:);
Cs2=2*Cs*SIG;Ss2=2*Ss*SIG;

Seta=zeros(ns,nx);Seta(idx,:)=eye(nx);
idsh=Seta(:,idshock);

if plotirf==1
    vars    = [6 3 4]; % choose default vars: y c l
    random=0;
    if random==1; periods=300; eee=randn(1,periods);
    else periods=60; eee=[1 zeros(1,periods-1)];end

    [ir1 ir2] = easyirf2(C,S,CS2,SS2,Cs2,Ss2,sig,eee,idsh);
    
    draw(real(ir1),txt,vars,periods,sig,eta);
    draw(real(ir2),txt,vars,periods,sig,eta); % Second order accurate IRFs
    invest=[ir1(5,:);ir2(5,:)]';
    drawi(invest,rho,gma); % draw irfs of investment separately.
end

%============== Plot Eigenvalues ====================

if ploteig==1 % plot the eigenvalues under different externality levels (eta)
    neta=1000;
    eigv=zeros(2,neta);
    etav=linspace(0,0.9,neta);
    for ieta=1:neta
        eigv(:,ieta)=eigvalue(etav(ieta));
    end
    eigv=real(eigv);
    idcom=find(diff(eigv)==0);idcom(2:end)=[]; % find the first complex eigenvalue
    lam1=[eigv(1,:);etav];
    lam2=[eigv(2,:);etav];
    
    figure;
    plot(lam1(2,1:idcom-1),lam1(1,1:idcom-1),'b');hold on;
    plot(lam1(2,idcom:end),lam1(1,idcom:end),'r--');
    figure;
    lam2(:,[532:535])=[]; % drop several big numbers.
    plot(lam2(2,1:idcom-5),lam2(1,1:idcom-5),'b');hold on;
    plot(lam2(2,idcom-4:end),lam2(1,idcom-4:end),'r--');
end

%============== Plot Coefficients ====================

if plotcoefs==1 
% Plot the coefficients for shock^2 and shock*K. 
% Use sunspot shocks for indeterminate cases.
    neta=100;
    etav=linspace(0,0.9,neta);
    coefv=zeros(4,neta);
    for ieta=1:neta
        eta=etav(ieta);
        a1=alf*(1+eta);b1=(1-alf)*(1+eta);
        
        ky = (bta*alf)/(1-bta*(1-dlt)); cy = 1-dlt*ky; %c/y ratio
        l= ((1-alf)/cy)^(1/(1+gma));
        k=(l^b1*ky)^(1/(1-a1));
        y = k/ky; c = cy*y; I = y-c;a = 1;
        
        a = log(a); k = log(k); l = log(l);
        c = log(c); y = log(y); I = log(I);
        
        symb=[v,param];nsymb=eval(symb);
        [A1, B1, B2, B2p] = derive(f,v,vp,symb,nsymb);
        
        % 2. Solve the system by linear method based on LS.
        idc=3:6;idx=1;
        [S S2 Ss C C2 Cs] = esolve(A1, B1, B2, B2p, idc, idx);
        indet=0;
        if isempty(S)
            indet=1;idc=4:6;idx=[1 3];
            [S S2 Ss C C2 Cs]=esolve(A1, B1, B2, B2p, idc, idx);
            coefv(:,ieta)=[S2(3,[6 5]) C2(1,[6 5])]';% store the coefficients
        else
            coefv(:,ieta)=[C2(1,1:2) C2(2,1:2)]';% store the coefficients
        end
    end
    coefv=real(coefv);
    figure;
    stem(coefv([1 3],:)',':');legend('Consumption','Labor','Location','SouthEast');
    figure;
    stem(coefv([2 4],:)',':');legend('Consumption','Labor','Location','NorthEast');
end

%==============Accuracy Test====================
%
if atest==1
    % Euler Equation Errors (eta=0.4)
    nk=20;na=20;
    
    kk=linspace(-1,1,nk);
    shock=linspace(-0.3,0.3,na);
        
    reska1=zeros(nk,na); % initialize residual matrix. 
    reska2=reska1;
    for ia=1:na
        state=[shock(ia)*ones(1,nk);kk];
        for ik=1:nk
            reska1(ik,ia)=ee1(state(:,ik),C,S,sig,alf,bta,dlt,eta,tao,k,c,l);
            reska2(ik,ia)=ee2(state(:,ik),CS2,SS2,C,S,Cs2,Ss2,sig,alf,bta,dlt,eta,tao,k,c,l);
        end
    end

    figure;
    surf(shock,kk,log10(abs(reska1)));
    hold on;
%     figure(2)
    surf(shock,kk,log10(abs(reska2)));
end

if atesteta==1
    % Euler Equation Errors (shock=0)
    nk=20;na=20;neta=50;
    kk=linspace(-1,1,nk);
    etav=linspace(0,0.9,neta);

    reske1=zeros(nk,neta);
    reske2=reske1;
    
    for ieta=1:neta % solve the model again for different eta.
        eta=etav(ieta);
        a1=alf*(1+eta);b1=(1-alf)*(1+eta);

        ky = (bta*alf)/(1-bta*(1-dlt)); cy = 1-dlt*ky; %c/y ratio

        l= ((1-alf)/cy)^(1/(1+gma));
        k=(l^b1*ky)^(1/(1-a1)); 
        y = k/ky; c = cy*y; I = y-c;a = 1; 

        a = log(a); k = log(k); l = log(l); 
        c = log(c); y = log(y); I = log(I);

        symb=[v,param];nsymb=eval(symb);

        [A1, B1, B2, B2p] = derive(f,v,vp,symb,nsymb);

        idc=3:6;idx=1; 
        [S S2 Ss C C2 Cs] = esolve(A1, B1, B2, B2p, idc, idx);
        indet=0;
        if isempty(S)
            indet=1;
            idc=4:6;idx=[1 3];
            [S S2 Ss C C2 Cs]=esolve(A1, B1, B2, B2p, idc, idx);
        end
        S=real(S);S2=real(S2);Ss=real(Ss);
        C=real(C);C2=real(C2);Cs=real(Cs);

        idshock=1;
        nc=length(idc);ns=nv-nc;nx=length(idx);
        % rearrange the coefficient matrices.
        [xL xC]=transformer(ns);[xl xc]=transformer(nx);
        T=eye(ns)+ones(ns);T=xC*T(:);
        SS2 = reshape(xL*(S2'.*(T*ones(1,ns))),ns,ns,ns);
        CS2 = reshape(xL*(C2'.*(T*ones(1,nc))),ns,ns,nc);

        SIG=eye(nx);SIG=xc*SIG(:);
        Cs2=2*Cs*SIG;Ss2=2*Ss*SIG;

        if indet==1
            state=[zeros(1,nk);kk;zeros(1,nk)];% shocks=0 
            for ik=1:nk
                reske1(ik,ieta)=eei1(state(:,ik),C,S,sig,alf,bta,dlt,eta,tao,k,c,l);
                reske2(ik,ieta)=eei2(state(:,ik),CS2,SS2,C,S,Cs2,Ss2,sig,alf,bta,dlt,eta,tao,k,c,l);
            end
        else
            for ik=1:nk
                state=[zeros(1,nk);kk];
                reske1(ik,ieta)=ee1(state(:,ik),C,S,sig,alf,bta,dlt,eta,tao,k,c,l);
                reske2(ik,ieta)=ee2(state(:,ik),CS2,SS2,C,S,Cs2,Ss2,sig,alf,bta,dlt,eta,tao,k,c,l);
            end
        end
    end

    figure;
    surf(etav,kk,log10(abs(reske1)));
    hold on;
    surf(etav,kk,log10(abs(reske2)));

    % Euler Equation Errors (k=0)
    k=0;
    shock=linspace(-0.3,0.3,na);
    etav=linspace(0,0.9,neta);

    resae1=zeros(na,neta);
    resae2=resae1;
    
    for ieta=1:neta
        eta=etav(ieta);
        a1=alf*(1+eta);b1=(1-alf)*(1+eta);

        ky = (bta*alf)/(1-bta*(1-dlt)); cy = 1-dlt*ky; %c/y ratio

        l= ((1-alf)/cy)^(1/(1+gma));
        k=(l^b1*ky)^(1/(1-a1)); 
        y = k/ky; c = cy*y; I = y-c;a = 1; 

        a = log(a); k = log(k); l = log(l); 
        c = log(c); y = log(y); I = log(I);

        symb=[v,param];nsymb=eval(symb);

        [A1, B1, B2, B2p] = derive(f,v,vp,symb,nsymb);

        idc=3:6;idx=1; 
        [S S2 Ss C C2 Cs] = esolve(A1, B1, B2, B2p, idc, idx);
        indet=0;
        if isempty(S)
            indet=1;
            idc=4:6;idx=[1 3];
            [S S2 Ss C C2 Cs]=esolve(A1, B1, B2, B2p, idc, idx);
        end
        S=real(S);S2=real(S2);Ss=real(Ss);
        C=real(C);C2=real(C2);Cs=real(Cs);

        idshock=1;
        nc=length(idc);ns=nv-nc;nx=length(idx);
        % rearrange the coefficient matrices.
        [xL xC]=transformer(ns);[xl xc]=transformer(nx);
        T=eye(ns)+ones(ns);T=xC*T(:);
        SS2 = reshape(xL*(S2'.*(T*ones(1,ns))),ns,ns,ns);
        CS2 = reshape(xL*(C2'.*(T*ones(1,nc))),ns,ns,nc);

        SIG=eye(nx);SIG=xc*SIG(:);
        Cs2=2*Cs*SIG;Ss2=2*Ss*SIG;
        if indet==1
            state=[shock;zeros(2,na)]; % k=0
            for ia=1:na
                resae1(ia,ieta)=eei1(state(:,ia),C,S,sig,alf,bta,dlt,etav(ieta),tao,k,c,l);
                resae2(ia,ieta)=eei2(state(:,ia),CS2,SS2,C,S,Cs2,Ss2,sig,alf,bta,dlt,etav(ieta),tao,k,c,l);
            end
        else
            state=[shock;zeros(1,na)];
            for ia=1:na
                resae1(ia,ieta)=ee1(state(:,ia),C,S,sig,alf,bta,dlt,etav(ieta),tao,k,c,l);
                resae2(ia,ieta)=ee2(state(:,ia),CS2,SS2,C,S,Cs2,Ss2,sig,alf,bta,dlt,etav(ieta),tao,k,c,l);
            end
        end
    end

    figure;
    surf(etav,shock,log10(abs(resae1)));
    hold on;
    surf(etav,shock,log10(abs(resae2)));
end
